/* Fix an issue between POSIX and C99 */
#if __STDC_VERSION__ >= 199901L
    #define _XOPEN_SOURCE 600
#else
    #define _XOPEN_SOURCE 500
#endif

#include "utils/time.h"
#include <limits.h>
#include <time.h>

#include "utils/logger.h"

#define NANO                    1000000000
#define MICRO                   1000000
#define MILLI                   1000

#define sleep(value, UNIT) {                                                    \
        struct timespec t;                                                      \
        int err;                                                                \
                                                                                \
        if (clock_gettime(CLOCK_MONOTONIC, &t) < 0) {                           \
                eerror("Failed to get time");                                   \
                return -1;                                                      \
        }                                                                       \
                                                                                \
        while (value > UNIT) {                                                  \
                value -= UNIT;                                                  \
                t.tv_sec++;                                                     \
        }                                                                       \
                                                                                \
        t.tv_nsec += value * (NANO / UNIT);                                     \
                                                                                \
        while (t.tv_nsec > NANO) {                                              \
                t.tv_nsec -= NANO;                                              \
                t.tv_sec++;                                                     \
        }                                                                       \
                                                                                \
        err = clock_nanosleep(CLOCK_MONOTONIC, TIMER_ABSTIME, &t, NULL);        \
                                                                                \
        if (err != 0 && err != EINTR) {                                         \
                error("Failed to sleep");                                       \
                return -1;                                                      \
        }                                                                       \
                                                                                \
        return 0;                                                               \
}

int sleep_ns(unsigned long ns) sleep(ns, NANO)
int sleep_us(unsigned long us) sleep(us, MICRO)
int sleep_ms(unsigned long ms) sleep(ms, MILLI)

#define time(UNIT) { \
        struct timespec t;                                                      \
                                                                                \
        if (clock_gettime(CLOCK_MONOTONIC, &t) < 0) {                           \
                eerror("Failed to get time");                                   \
                return -1;                                                      \
        }                                                                       \
                                                                                \
        return ((t.tv_sec * UNIT) + ((t.tv_nsec * UNIT) / NANO)) & LONG_MAX;    \
}

long time_ns(void) time(NANO)
long time_us(void) time(MICRO)
long time_ms(void) time(MILLI)

/* return duration in seconds */
/* get startTime using clock_gettime(CLOCK_REALTIME, &startTime); */
double get_duration_since(struct timespec* startTime)
{
	struct timespec now;
	long nresult;
	double result;

	clock_gettime(CLOCK_REALTIME, &now);
	result = (double)(now.tv_sec - startTime->tv_sec);
	nresult = now.tv_nsec - startTime->tv_nsec;
	return result + ((double)nresult) / 1000000000;
}
