#!/bin/sh
CONFIG="debug_conf.json"

RADIO=0
FREQUENCY0=867500000
FREQUENCY1=868500000
BANDWIDTH=125
SPREADING_FACTOR=0

usage() {
	cat << EOF
usage: ${0##*/} [OPTIONS]

  -r, --radio <radio_to_use>                 (0 or 1) (default ${RADIO})
  -f0, --radio0_frequency <frequency_in_hz>  (default ${FREQUENCY0})
  -f1, --radio1_frequency <frequency_in_hz>  (default ${FREQUENCY1})
  -b, --bandwidth <bandwidth_in_Khz>         (default ${BANDWIDTH})
  -s, --sf <spreading_factor>                (7-12 or 0 for dynamic) (default ${SPREADING_FACTOR})
EOF
}

while test $# -gt 0
do
	case $1 in
		-f0|--radio0_frequency)
			FREQUENCY0="${2}"
			shift ;;
		-f1|--radio1_frequency)
			FREQUENCY1="${2}"
			shift ;;
		-b|--bandwidth)
			BANDWIDTH="${2}"
			shift ;;
		-s|--sf)
			SPREADING_FACTOR="$2"
			shift ;;
		-r|--radio)
			RADIO="$2"
			shift ;;
		*)	usage && exit 1 ;;
	esac
	shift
done

cat > ${CONFIG} << EOF
{
	"SX1301_conf": {
		"lorawan_public": true,
		"clksrc": 1,
		"radio_0": {
			"enable": true,
			"type": "SX1257",
			"freq": ${FREQUENCY0},
			"rssi_offset": -166.0,
			"tx_enable": false
		},
		"radio_1": {
			"enable": true,
			"type": "SX1257",
			"freq": ${FREQUENCY1},
			"rssi_offset": -166.0,
			"tx_enable": false
		},
EOF

if [ ${SPREADING_FACTOR} -eq 0 ]
then
	cat >> ${CONFIG} << EOF
		"chan_multiSF_0": {
			"enable": true,
			"radio": ${RADIO},
			"if": 0
		}
	},
EOF
else
	cat >> ${CONFIG} << EOF
		"chan_Lora_std": {
			"enable": true,
			"radio": ${RADIO},
			"if": 0,
			"bandwidth": $(( ${BANDWIDTH} * 1000 )),
			"spread_factor": ${SPREADING_FACTOR}
		}
	},
EOF
fi

if [ -f /sys/class/net/eth0/address ]
then # Stations
EUI=$(awk -F: '{ printf "7276FF00%.2X%.2X%.2X%.2X", "0x"$3, "0x"$4, "0x"$5, "0x"$6 }' /sys/class/net/eth0/address)
elif [ -f /sys/fsl_otp/HW_OCOTP_GP1 ]
then # iFemtocell
EUI=$(awk '{ printf "7276FF00%.8X", $1 }' /sys/fsl_otp/HW_OCOTP_GP1)
else # others
EUI="AA555A0000000000"
fi

cat >> ${CONFIG} << EOF
	"gateway_conf": {
		"gateway_ID": "${EUI}"
	}
}
EOF

util_fpga_start
if [ $? -ne 0 ]
then
	echo "********************************************************************"
	echo " WARNING: Unable to connect FPGA, binary is executed in background ?"
	echo "********************************************************************"
fi

./pkt_logger &

sleep 6 && echo "**  Waiting for packets ...."
tail -f pktlog.csv | awk -F, '{print; print "RSSI= " $14 ", SNR= "$15}'
